#!/usr/bin/env python

import psycopg2, sys, time
from optparse import OptionParser
import traceback, pickle,  math

cur =  None
handled = 0
penaltyscore = 1
resultfile = None

def main():
    global cur, filelength, truelocations, estimatedlocations, penaltyscore, dumpcell, dumpdata, resultfile
    
    parser = OptionParser(version='%prog 1.0', usage="usage: %prog [options] filename", description='Analyze algorithm 1')
    parser.add_option('-p', '--password', action='store', type='string', dest='password', default='', help='Set database password')
    parser.add_option('-u', '--username', action='store', type='string', dest='username', default='', help='Set database username')
    parser.add_option('-n', '--hostname', action='store', type='string', dest='hostname', default='localhost', help='Set database hostname')
    parser.add_option('-d', '--dbname', action='store', type='string', dest='dbname', default='gloc', help='Set database name')
    parser.add_option('-a', '--penalty', action='store', type='int', dest='penaltyscore', default=1, help='How high a penalty a missing network / cell should incure')
    parser.add_option('-r', '--resultfile', action='store', type='string', dest='resultfile', default='', help='File to save result')

    (options, args) = parser.parse_args()

    fid =  None

    if len(args) != 1:
        parser.error("You must specify file to be parsed")

    if not options.resultfile:
        parser.error("You must specify result")

    try: resultfile = open(options.resultfile, 'w')
    except: parser.error("Error opening resultfile")    

    try:
        conn = psycopg2.connect("dbname='%s' user='%s' host='%s' password='%s'"%(options.dbname, options.username, options.hostname, options.password))
        cur = conn.cursor()

        infile = open(args[0], 'r')

        infile.seek(0)
        
        linecnt = 0
        results = []
        
        for line in infile: 
            result = eval(line)
            if result:
                results.append(linecnt)
            linecnt += 1

    except:
        print "ERROR %s"%(sys.exc_info()[1]);
        traceback.print_exc()

    print "\nCount: %s" % len(results)        

    pickle.dump(results, resultfile)
    resultfile.close()

def locationQuery(timestamp, lat, lon, imei, values):
    global cur, filelength, handled, truelocations, estimatedlocations, penaltyscore, dumpdata, dumpcell, resultfile

    t0 = time.time()
    if len(values) > 1: #skip fingerprints only containing a single serving cell (they make no sense for this algo)
        candidateprints = []
        matches = []
        for entry in values:
            #if entry['type'] == 'gsm':
            #    cur.execute("SELECT cid FROM cells WHERE la =%s AND mcc =%s AND mnc =%s AND cellid =%s;" % (entry['la'], entry['mcc'], entry['mnc'], entry['cellid']))
            #    result =  cur.fetchall()
            #    cur.execute("SELECT fid FROM cellobservations WHERE cid =%s;" % result[0])
            #    result = cur.fetchall()
            #    if len(result) > 1:
            #        for res in result:
            #            cur.execute("SELECT gps_time = TIMESTAMP 'epoch' + %s * INTERVAL '1 second' FROM fingerprints WHERE fid =%s;" % (timestamp, res[0]))
            #            gpst = cur.fetchall()
            #            if not gpst[0][0] and res[0] not in candidateprints: #skip the fingerprint that has been added using this entry
            #                candidateprints.append(res[0])
            #else:
            if entry['type'] == 'neighbour': #For this algorithm we are only interested in fingerprints with more than 1 measurement,  and since the database doesn't contain any fingerprints with multiple gsm measurements yet we can simply forget about looking at gsm fingerprints here for now
                if 'psc' in entry: 
                    cur.execute("SELECT fid FROM neighbouringcells WHERE psc= '%s';" % entry['psc'])
                else:
                    cur.execute("SELECT cid FROM cells WHERE la =%s AND mcc =%s AND mnc =%s AND cellid =%s;" % (entry['la'], entry['mcc'], entry['mnc'], entry['cellid']))
                    cidres =  cur.fetchall()
                    cur.execute("SELECT fid FROM neighbouringcells WHERE cid= '%s';" % cidres[0])
                result = cur.fetchall()
                if len(result) > 1:
                    for res in result:
                        cur.execute("SELECT gps_time = TIMESTAMP 'epoch' + %s * INTERVAL '1 second' FROM fingerprints WHERE fid =%s;" % (timestamp, res[0]))
                        gpst = cur.fetchall()
                        if not gpst[0][0] and res[0] not in candidateprints:
                            return True
            elif entry['type'] ==  'wlan':
                cur.execute("SELECT fid FROM wlanobservations WHERE bssid = '%s';" % entry['bssid'])
                result = cur.fetchall()
                if len(result) > 1:
                    for res in result:
                        cur.execute("SELECT gps_time = TIMESTAMP 'epoch' + %s * INTERVAL '1 second' FROM fingerprints WHERE fid =%s;" % (timestamp, res[0]))
                        gpst = cur.fetchall()
                        if not gpst[0][0] and res[0] not in candidateprints:
                            return True
    return False

if __name__ == "__main__":
    main()

