#!/usr/bin/env python

import psycopg2, sys, time
from optparse import OptionParser
import traceback, pickle,  math

cur =  None
filelength = 0
handled = 0
truelocations = []
estimatedlocations = []
penaltyscore = 1
dumpcell = 0
dumpdata = []
resultfile = None

def distance(lat1,  lon1,  lat2,  lon2):
    radius =   6371 # km

    dlat =   math.radians(lat2 -  lat1)
    dlon =   math.radians(lon2 -  lon1)
    a =   math.sin(dlat /  2) *   math.sin(dlat /  2) +   math.cos(math.radians(lat1)) *   math.cos(math.radians(lat2)) *   math.sin(dlon /  2) *   math.sin(dlon /  2)
    c =   2 *   math.atan2(math.sqrt(a),   math.sqrt(1 -  a))
    d =   radius *   c
    return d *  1000

def main():
    global cur, filelength, truelocations, estimatedlocations, penaltyscore, dumpcell, dumpdata, resultfile
    
    parser = OptionParser(version='%prog 1.0', usage="usage: %prog [options] filename", description='Analyze algorithm 1')
    parser.add_option('-p', '--password', action='store', type='string', dest='password', default='', help='Set database password')
    parser.add_option('-u', '--username', action='store', type='string', dest='username', default='', help='Set database username')
    parser.add_option('-n', '--hostname', action='store', type='string', dest='hostname', default='localhost', help='Set database hostname')
    parser.add_option('-d', '--dbname', action='store', type='string', dest='dbname', default='gloc', help='Set database name')
    parser.add_option('-t', '--truefile', action='store', type='string', dest='truefile', default='', help='File to save true locations to')
    parser.add_option('-f', '--dumpfile', action='store', type='string', dest='dumpfile', default=None, help='File to dump one cell to')
    parser.add_option('-e', '--estimatefile', action='store', type='string', dest='estimatefile', default='', help='File to save estimated locations to')
    parser.add_option('-a', '--penalty', action='store', type='int', dest='penaltyscore', default=1, help='How high a penalty a missing network / cell should incure')
    parser.add_option('-c', '--dumpcell', action='store', type='int', dest='dumpcell', default=0, help='The number of the cell to dump all information on to a file')
    parser.add_option('-r', '--resultfile', action='store', type='string', dest='resultfile', default='', help='File to save result')

    (options, args) = parser.parse_args()

    fid =  None

    if len(args) != 1:
        parser.error("You must specify file to be parsed")

    if not options.truefile:
        parser.error("You must specify truefile")

    if not options.resultfile:
        parser.error("You must specify result")

    if not options.estimatefile:
        parser.error("You must specify estimatefile")

    try: truefile = open(options.truefile, 'w')
    except: parser.error("Error opening truefile")

    try: estimatefile = open(options.estimatefile, 'w')
    except: parser.error("Error opening estimatefile")

    try: resultfile = open(options.resultfile, 'w')
    except: parser.error("Error opening resultfile")    

    dumpfile =  None
    if options.dumpfile:
        try: dumpfile = open(options.dumpfile, 'w')
        except: parser.error("Error opening dumpfile")

    dumpcell = options.dumpcell
    penaltyscore = options.penaltyscore

    try:
        conn = psycopg2.connect("dbname='%s' user='%s' host='%s' password='%s'"%(options.dbname, options.username, options.hostname, options.password))
        cur = conn.cursor()

        infile = open(args[0], 'r')

        for line in infile:
            filelength += 1

        infile.seek(0)
        
        for line in infile: 
            result = eval(line)

    except:
        print "ERROR %s"%(sys.exc_info()[1]);
        traceback.print_exc()

    pickle.dump(truelocations, truefile)
    pickle.dump(estimatedlocations, estimatefile)

    if dumpfile:
        pickle.dump(dumpdata, dumpfile)

    truefile.close()
    estimatefile.close()
    resultfile.close()

def locationQuery(timestamp, lat, lon, imei, values):
    global cur, filelength, handled, truelocations, estimatedlocations, penaltyscore, dumpdata, dumpcell, resultfile

    t0 = time.time()
    if len(values) > 1: #skip fingerprints only containing a single serving cell (they make no sense for this algo)
        candidateprints = []
        matches = []
        for entry in values:
            #if entry['type'] == 'gsm':
            #    cur.execute("SELECT cid FROM cells WHERE la =%s AND mcc =%s AND mnc =%s AND cellid =%s;" % (entry['la'], entry['mcc'], entry['mnc'], entry['cellid']))
            #    result =  cur.fetchall()
            #    cur.execute("SELECT fid FROM cellobservations WHERE cid =%s;" % result[0])
            #    result = cur.fetchall()
            #    if len(result) > 1:
            #        for res in result:
            #            cur.execute("SELECT gps_time = TIMESTAMP 'epoch' + %s * INTERVAL '1 second' FROM fingerprints WHERE fid =%s;" % (timestamp, res[0]))
            #            gpst = cur.fetchall()
            #            if not gpst[0][0] and res[0] not in candidateprints: #skip the fingerprint that has been added using this entry
            #                candidateprints.append(res[0])
            #else:
            if entry['type'] == 'neighbour': #For this algorithm we are only interested in fingerprints with more than 1 measurement,  and since the database doesn't contain any fingerprints with multiple gsm measurements yet we can simply forget about looking at gsm fingerprints here for now
                if 'psc' in entry: 
                    cur.execute("SELECT fid FROM neighbouringcells WHERE psc= '%s';" % entry['psc'])
                else:
                    cur.execute("SELECT cid FROM cells WHERE la =%s AND mcc =%s AND mnc =%s AND cellid =%s;" % (entry['la'], entry['mcc'], entry['mnc'], entry['cellid']))
                    cidres =  cur.fetchall()
                    cur.execute("SELECT fid FROM neighbouringcells WHERE cid= '%s';" % cidres[0])
                result = cur.fetchall()
                if len(result) > 1:
                    for res in result:
                        cur.execute("SELECT gps_time = TIMESTAMP 'epoch' + %s * INTERVAL '1 second' FROM fingerprints WHERE fid =%s;" % (timestamp, res[0]))
                        gpst = cur.fetchall()
                        if not gpst[0][0] and res[0] not in candidateprints:
                            candidateprints.append(res[0])
            elif entry['type'] ==  'wlan':
                cur.execute("SELECT fid FROM wlanobservations WHERE bssid = '%s';" % entry['bssid'])
                result = cur.fetchall()
                if len(result) > 1:
                    for res in result:
                        cur.execute("SELECT gps_time = TIMESTAMP 'epoch' + %s * INTERVAL '1 second' FROM fingerprints WHERE fid =%s;" % (timestamp, res[0]))
                        gpst = cur.fetchall()
                        if not gpst[0][0] and res[0] not in candidateprints:
                            candidateprints.append(res[0])
        print "Time 1: ", time.time() - t0
        t0 = time.time()
                                
        if len(candidateprints) > 0:
            print "Candidates: ", len(candidateprints)
            for finger in candidateprints:
                weight = 0
                cur.execute("SELECT cid, rxlev, psc, rscp FROM neighbouringcells WHERE fid =%s;" % finger)
                result =  cur.fetchall()
                cur.execute("SELECT cid, rxlev FROM cellobservations WHERE fid =%s;" % finger)
                result2 =  cur.fetchall()
                cur.execute("SELECT bssid, rxlev FROM wlanobservations WHERE fid =%s;" % finger)
                result3 =  cur.fetchall()
                found = 0
                for entry in values:
                    if entry['type'] == 'neighbour':
                        for fing in result:
                            if fing[0] and 'cellid' in entry:
                                cur.execute("SELECT cid FROM cells WHERE la =%s AND mcc =%s AND mnc =%s AND cellid =%s;" % (entry['la'], entry['mcc'], entry['mnc'], entry['cellid']))
                                cidres =  cur.fetchall()
                                if fing[0] == cidres[0][0]: 
                                    weight += ( -int(entry['rxlev']) + -fing[1]) ** 2
                                    found += 1
                            elif fing[2] and 'psc' in entry:
                                if int(fing[2]) == int(entry['psc']): 
                                    weight += ( int(entry['rscp']) + fing[3]) ** 2
                                    found += 1
                    elif entry['type'] == 'wlan':
                        for fing in result3:
                            if fing[0] == entry['bssid']:
                                weight += ( -int(entry['rxlev']) + -fing[1]) ** 2
                                found += 1
                    elif entry['type'] == 'gsm':
                        cur.execute("SELECT cid FROM cells WHERE la =%s AND mcc =%s AND mnc =%s AND cellid =%s;" % (entry['la'], entry['mcc'], entry['mnc'], entry['cellid']))
                        cidres =  cur.fetchall()
                        for fing in result2:
                            if len(cidres) > 1: 
                                if fing[0] == cidres[0][0]:
                                    weight += ( int(entry['rxlev']) - fing[1]) ** 2
                                    found += 1
                penalty = len(values) - found + len(result) + len(result2) + len(result3) - found #for now we just increase penalty by one for each missing network / cell
                #print "Fingerprint ", finger, " gets score: ", weight - penalty #printing only for debuging,  too slow if not

                matches.append([finger, weight - (penalty * penaltyscore)])

            print ""
            print "Time 2: ", time.time() - t0
            t0 = time.time()
            
            endresult =  [-1000000000,-100000000]
            for match in matches: #TODO:  This should really be done using some sort of sort algorithm
                if match[1] > endresult[1]:
                    endresult = match

            print "Time 3: ", time.time() - t0
            t0 = time.time()
                
            cur.execute("SELECT latitude, longitude FROM gps WHERE fid =%s;" % endresult[0])
            position = cur.fetchall()
        
            if handled == dumpcell:
                for pt in matches:
                    cur.execute("SELECT latitude, longitude FROM gps WHERE fid =%s;" % pt[0])
                    ptpos =  cur.fetchall()
                    dumpdata.append((ptpos[0][0], ptpos[0][1], [1 - (float(pt[1]) / float(endresult[1])),  float(pt[1]) / float(endresult[1]),  0.0,  1]))
            
                dumpdata.append((position[0][0], position[0][1], [0.0,  1.0,  1.0,  1]))
                dumpdata.append((lat, lon, [0.0,  0.0,  1.0,  1]))

            diff =  distance(lat, lon, position[0][0], position[0][1])
            resultfile.write("choose: %s score: %s true: %s,%s estimated: %s,%s diff (meters): %s\n" % (endresult[0], endresult[1], lat, lon, position[0][0], position[0][1], diff))
            print "Choose cell: ", endresult[0], " with score: ", endresult[1]
            print "True position:      ", lat, " ", lon
            print "Estimated position: ", position[0][0], position[0][1]
            print "Finished: ", (handled * 100 / filelength), "% of file"
            print " -------------------------------------------------------- "

            truelocations.append((lat, lon))
            estimatedlocations.append((position[0][0], position[0][1]))

    handled += 1

        #Illustrations to be made (so we need to be able to dump)
        #real track vs estimated track
        #real position, all candidates and choosen candidate for a single cell (should color code the candidates depeding uppon how close they are to a match)
        #TODO: We could actually use this on single cell stuff too,  but we would only be relying on the penalty,  nothing else
        #TODO: Include option to dump all information for all cells to file?
        #TODO: Put in option in glocalizer for specifying radius when rendering stuff

if __name__ == "__main__":
    main()

