#!/usr/bin/env python

import psycopg2, sys
from optparse import OptionParser
import traceback, pickle
import Polygon, math

cur =  None
filelength = 0
handled = 0
truelocations = []
estimatedlocations = []
dumpcell = 0
dumpfile = None
resultfile = None

def distance(lat1,  lon1,  lat2,  lon2):
    radius =   6371 # km

    dlat =   math.radians(lat2 -  lat1)
    dlon =   math.radians(lon2 -  lon1)
    a =   math.sin(dlat /  2) *   math.sin(dlat /  2) +   math.cos(math.radians(lat1)) *   math.cos(math.radians(lat2)) *   math.sin(dlon /  2) *   math.sin(dlon /  2)
    c =   2 *   math.atan2(math.sqrt(a),   math.sqrt(1 -  a))
    d =   radius *   c
    return d *  1000

def main():
    global cur, filelength, truelocations, estimatedlocations, dumpcell, dumpfile, resultfile
    
    parser = OptionParser(version='%prog 1.0', usage="usage: %prog [options] filename", description='Analyze algorithm 1')
    parser.add_option('-p', '--password', action='store', type='string', dest='password', default='', help='Set database password')
    parser.add_option('-u', '--username', action='store', type='string', dest='username', default='', help='Set database username')
    parser.add_option('-n', '--hostname', action='store', type='string', dest='hostname', default='localhost', help='Set database hostname')
    parser.add_option('-d', '--dbname', action='store', type='string', dest='dbname', default='gloc', help='Set database name')
    parser.add_option('-t', '--truefile', action='store', type='string', dest='truefile', default='', help='File to save true locations to')
    parser.add_option('-f', '--dumpfile', action='store', type='string', dest='dumpfile', default=None, help='File to dump one cell to')
    parser.add_option('-e', '--estimatefile', action='store', type='string', dest='estimatefile', default='', help='File to save estimated locations to')
    parser.add_option('-c', '--dumpcell', action='store', type='int', dest='dumpcell', default=0, help='The number of the cell to dump all information on to a file')
    parser.add_option('-r', '--resultfile', action='store', type='string', dest='resultfile', default='', help='File to save result')
    parser.add_option('-o', '--forcefile', action='store', type='string', dest='forcefile', default=None, help='Only analyze the lines specified in this file')

    (options, args) = parser.parse_args()

    fid =  None

    if len(args) != 1:
        parser.error("You must specify file to be parsed")

    if not options.truefile:
        parser.error("You must specify truefile")

    if not options.resultfile:
        parser.error("You must specify result")

    if not options.estimatefile:
        parser.error("You must specify estimatefile")

    try: truefile = open(options.truefile, 'w')
    except: parser.error("Error opening truefile")

    try: estimatefile = open(options.estimatefile, 'w')
    except: parser.error("Error opening estimatefile")

    try: resultfile = open(options.resultfile, 'w')
    except: parser.error("Error opening resultfile")

    cellist = None

    if options.forcefile:
        try: forcefile = open(options.forcefile, 'r')
        except: parser.error("Error opening forcefile")
        cellist =  pickle.load(forcefile)
        forcefile.close()

    dumpfile = options.dumpfile
    dumpcell = options.dumpcell

    linecnt = 0

    try:
        conn = psycopg2.connect("dbname='%s' user='%s' host='%s' password='%s'"%(options.dbname, options.username, options.hostname, options.password))
        cur = conn.cursor()

        infile = open(args[0], 'r')

        for line in infile:
            filelength += 1

        infile.seek(0)
        
        for line in infile:
            goodtogo = True
            if cellist:
                if linecnt not in cellist:
                    goodtogo =  False
            if goodtogo: 
                result = eval(line)

            linecnt += 1

    except:
        print "ERROR %s"%(sys.exc_info()[1]);
        traceback.print_exc()

    pickle.dump(truelocations, truefile)
    pickle.dump(estimatedlocations, estimatefile)

    truefile.close()
    estimatefile.close()
    resultfile.close()

def locationQuery(timestamp, lat, lon, imei, values):
    global cur, filelength, handled, truelocations, estimatedlocations, dumpfile, dumpcell, resultfile
    
    if len(values) > 1: #skip fingerprints only containing a single serving cell (they make no sense for this algo)
        areas = []
        for entry in values:
            if entry['type'] == 'gsm':
                cur.execute("SELECT cid FROM cells WHERE la =%s AND mcc =%s AND mnc =%s AND cellid =%s;" % (entry['la'], entry['mcc'], entry['mnc'], entry['cellid']))
                cidres =  cur.fetchall()
                if cidres: 
                    cur.execute("SELECT area FROM algotestground.cellareas WHERE cid=%s AND rxlev=%s;" % (cidres[0][0], entry['rxlev']))
            elif entry['type'] == 'neighbour':
                if 'psc' in entry:
                    cur.execute("SELECT area FROM algotestground.neighbourareas3g WHERE psc=%s AND rscp=%s;" % (entry['psc'], entry['rscp']))
                elif 'cellid' in entry:
                    cur.execute("SELECT cid FROM cells WHERE la =%s AND mcc =%s AND mnc =%s AND cellid =%s;" % (entry['la'], entry['mcc'], entry['mnc'], entry['cellid']))
                    cidres =  cur.fetchall()
                    if cidres:
                        cur.execute("SELECT area FROM algotestground.neighbourareas WHERE cid=%s AND rxlev=%s;" % (cidres[0][0], entry['rxlev']))

            result = cur.fetchall()
            if len(result) > 0: 
                newpol = [(float(y[0]), float(y[1])) for y in [x.split(',') for x in result[0][0].split(';')]]
                if len(newpol) > 3: 
                    areas.append(Polygon.Polygon(newpol))
        if len(areas) > 0: 
            area = areas[0]
            if handled == dumpcell and dumpfile:
                try:
                    tullfile = open('%s' % dumpfile, 'w')
                    pickle.dump(areas[0][0], tullfile)
                    tullfile.close()
                except: print "gugg"
            points = {}
            cnt = 1
            penalty = 0
            #TODO: Need to check that the first one is an actual area not just a point or line
            for ar in areas[1:]:
                if ar[0][0] == ar[0][-1] and ar[0][1] == ar[0][-2]: 
                    for point in ar[0]:
                        if area.covers(Polygon.Polygon([(point), (point), (point), (point)])):
                            if point not in points:
                                points[point] = 0
                            else:
                                points[point] += 1
                        else:
                            penalty += 1
                else:
                    if handled == dumpcell and dumpfile:                
                        try:
                            tullfile = open('%s%s' % (dumpfile, cnt), 'w')
                            pickle.dump(ar[0], tullfile)
                            tullfile.close()
                        except: print "gugg"
                    cnt += 1
                    if area.overlaps(ar):
                        area = ar & area
                    else:
                        penalty += 1

#            bb = area.boundingBox() 
#            bx = (bb[0] + bb[1]) / 2
#            by = (bb[2] + bb[3]) / 2

            bx = 0
            by = 0
            pcnt = 0

            for point in area[0]:
                bx += point[0]
                by += point[1]
                pcnt += 1

            bx =  bx / pcnt
            by =  by / pcnt

            truelocations.append((lat, lon))
            estimatedlocations.append((bx, by))        
            diff =  distance(lat, lon, bx, by)
            resultfile.write("penalty: %s true: %s,%s estimated: %s,%s diff (meters): %s\n" % (penalty, lat, lon, bx, by, diff))
        
            print "True position:      ", lat, " ", lon
            print "Estimated position: ", bx, " ", by
            print "Finished: ", (handled * 100 / filelength), "% of file"
            print " -------------------------------------------------------- "

            if handled == dumpcell and dumpfile:
                try:
                    tullfile = open('%sfinalarea'%dumpfile, 'w')
                    pickle.dump(area[0], tullfile)
                    tullfile.close()
                except: print "gugg"
            
                try:
                    tullfile = open('%sestimatedpoint'%dumpfile, 'w')
                    pickle.dump([(bx, by)], tullfile)
                    tullfile.close()
                except: print "gugg"

                try:
                    tullfile = open('%struepoint'%dumpfile, 'w')
                    pickle.dump([(lat, lon)], tullfile)
                    tullfile.close()
                except: print "gugg"

    handled += 1


        #Illustrations to be made (so we need to be able to dump)
        #real track vs estimated track
        #real position, all candidates and choosen candidate for a single cell (should color code the candidates depeding uppon how close they are to a match)
        #TODO: Include option to dump all information for all cells to file?
        #TODO: Put in option in glocalizer for specifying radius when rendering stuff

if __name__ == "__main__":
    main()

