#!/usr/bin/env python

import psycopg2, sys
from optparse import OptionParser
import traceback

def main():

    parser = OptionParser(version='%prog 1.0', usage="usage: %prog [options] filename", description='Add logfile to gloc database.')
    parser.add_option('-p', '--password', action='store', type='string', dest='password', default='', help='Set database password')
    parser.add_option('-u', '--username', action='store', type='string', dest='username', default='', help='Set database username')
    parser.add_option('-n', '--hostname', action='store', type='string', dest='hostname', default='localhost', help='Set database hostname')
    parser.add_option('-d', '--dbname', action='store', type='string', dest='dbname', default='gloc', help='Set database name')
    parser.add_option('-i', '--imei', action='store', type='string', dest='imei', default='none', help='Device IMEI. Use if the logfile does not contain an IMEI. If the logfile contains an IMEI this option will be ignored.')

    (options, args) = parser.parse_args()

    fid =  None

    if len(args) != 1:
        parser.error("You must specify file to be parsed")

    try:
        conn = psycopg2.connect("dbname='%s' user='%s' host='%s' password='%s'"%(options.dbname, options.username, options.hostname, options.password))
        cur = conn.cursor()

        infile = open(args[0], 'r')
        info = infile.readline().rstrip('\r\n').split(",")

        if len(info)>1:
            options.imei = info[0]
            version = (',').join(info[1:])

        if options.imei == 'none':
            parser.error("No IMEI found in logfile. You must specify an IMEI")

        options.imei =  0 #quick hack

        cur.execute("SELECT string FROM equipmentcapabilities WHERE string='%s';"%version)
        result = cur.fetchall()

        if len(result)<1:
            cur.execute("INSERT into equipmentcapabilities(string) VALUES('%s') RETURNING string;"%version)
            result = cur.fetchall()
            conn.commit()
            print "Added new equipment string: %s"%version
        equipment = result[0][0]

        cur.execute("SELECT imei FROM equipment WHERE imei=%s;"%options.imei)
        result = cur.fetchall()
        if len(result)<1:
            cur.execute("INSERT INTO equipment VALUES(%s,'%s') RETURNING imei;"%(options.imei,equipment))
            result = cur.fetchall()
            conn.commit()
            print "Added new equipment IMEI: %s"%options.imei
        imei = result[0][0]

        cnt_fingerprints = 0
        cnt_gps = 0
        cnt_gsmn = 0
        cnt_gsm = 0
        cnt_wlan = 0

        
        for line in infile:
            #print line
            wlans = None
            entries = line.rstrip('\r\n').split(';')

            gpsdata = entries[0].split(',')
            gsmdata = entries[1].split(',')
            neighbours = entries[2].split('|')
            if len(entries) > 3:
                wlans = entries[3:]
                wlans =  [ x.split('|')[0] for x in wlans ]

            cur.execute("SELECT fid FROM fingerprints WHERE imei=%s AND gps_time=TIMESTAMP 'epoch' + %s * INTERVAL '1 ms';"%(imei,gpsdata[7]))
            result = cur.fetchall()
            if len(result)<1:
                cur.execute("INSERT INTO fingerprints (imei,gps_time) VALUES(%s,TIMESTAMP 'epoch' + %s * INTERVAL '1 ms') RETURNING fid;"%(imei,gpsdata[7]))
                result = cur.fetchall()
                conn.commit()
                cnt_fingerprints = cnt_fingerprints + 1
            fid = result[0][0]            
            
            cur.execute("SELECT gid FROM gps WHERE fid=%s;"%fid)
            result = cur.fetchall()
            if len(result)<1:
                del gpsdata[5]
                del gpsdata[5]
                gpsdata.append(str(fid))
                
                cur.execute("INSERT INTO gps (horizontal_accuracy, altitude, heading, latitude, longitude, speed, fid) VALUES(%s, %s, %s, %s, %s, %s, %s) RETURNING gid;" % (tuple(gpsdata)))
                result = cur.fetchall()
                conn.commit()
                cnt_gps = cnt_gps + 1
            gid = result[0][0]

            cur.execute("SELECT mcc FROM mcc WHERE mcc=%s;"%gsmdata[0])
            result = cur.fetchall()
            if len(result)<1:
                cur.execute("INSERT INTO mcc VALUES(%s,'n/a','na');"%gsmdata[0])
                conn.commit()
                print "Added new mcc: %s"%gsmdata[0]
                    
            cur.execute("SELECT mnc FROM mnc WHERE mcc=%s AND mnc=%s;"%(gsmdata[0], gsmdata[1]))
            result = cur.fetchall()
            if len(result)<1: 
                cur.execute("INSERT INTO mnc VALUES(%s,%s,'n/a');"%(gsmdata[0], gsmdata[1]))
                conn.commit()
                print "Added new mcn: %s %s"%(gsmdata[0], gsmdata[1])

            cur.execute("SELECT la FROM la WHERE mcc=%s AND mnc=%s AND la=%s;"%(gsmdata[0], gsmdata[1], gsmdata[2]))
            result = cur.fetchall()
            if len(result)<1:
                cur.execute("INSERT INTO la VALUES(%s,%s,%s,'n/a');"%(gsmdata[0], gsmdata[1], gsmdata[2]))
                conn.commit()
                print "Added new la: %s %s %s"%(gsmdata[0], gsmdata[1], gsmdata[2])
                    
            cur.execute("SELECT cid FROM cells WHERE mcc=%s AND mnc=%s AND la=%s AND cellid=%s;"%(gsmdata[0], gsmdata[1], gsmdata[2], gsmdata[3]))
            result = cur.fetchall()
            if len(result)<1:
                cur.execute("INSERT INTO cells (mcc,mnc,la,cellid,name) VALUES(%s,%s,%s,%s,'n/a') RETURNING cid;"%(gsmdata[0], gsmdata[1], gsmdata[2], gsmdata[3]))
                result = cur.fetchall()
                conn.commit()
                print "Added new cell: %s %s %s %s"%(gsmdata[0], gsmdata[1], gsmdata[2], gsmdata[3])
            cid = result[0][0]
                    
            cur.execute("SELECT *  FROM cellobservations where cid = %s AND fid = %s" % (cid,fid))
            result = cur.fetchall()
            if len(result) < 1: 
                cur.execute("INSERT INTO cellobservations (cid,rxlev,fid) VALUES(%s,%s,%s);"%(cid,gsmdata[4],fid))
                conn.commit()
                cnt_gsm = cnt_gsm + 1


            for neighbour in neighbours:
                ndata = neighbour.split(",")
                if len(ndata) < 3 and len(ndata) > 1:
                    cur.execute("SELECT *  FROM neighbouringcells where psc = %s AND fid = %s" % (ndata[0],fid))
                    result = cur.fetchall()
                    if len(result) < 1: 
                        cur.execute("INSERT INTO neighbouringcells (psc, rscp ,fid) VALUES(%s,%s,%s);"%(ndata[0],ndata[1],fid))
                        conn.commit()
                        cnt_gsmn = cnt_gsmn + 1
                elif len(ndata) > 2:
                    cur.execute("SELECT la FROM la WHERE mcc=%s AND mnc=%s AND la=%s;"%(gsmdata[0], gsmdata[1], ndata[0]))
                    result = cur.fetchall()
                    if len(result)<1:
                        cur.execute("INSERT INTO la VALUES(%s,%s,%s,'n/a');"%(gsmdata[0], gsmdata[1], ndata[0]))
                        conn.commit()
                        print "Added new la: %s %s %s"%(gsmdata[0], gsmdata[1], ndata[0])
                    
                    cur.execute("SELECT cid FROM cells WHERE mcc=%s AND mnc=%s AND la=%s AND cellid=%s;"%(gsmdata[0], gsmdata[1], ndata[0], ndata[1]))
                    result = cur.fetchall()
                    if len(result)<1:
                        cur.execute("INSERT INTO cells (mcc,mnc,la,cellid,name) VALUES(%s,%s,%s,%s,'n/a') RETURNING cid;"%(gsmdata[0], gsmdata[1], ndata[0], ndata[1]))
                        result = cur.fetchall()
                        conn.commit()
                        print "Added new cell: %s %s %s %s"%(gsmdata[0], gsmdata[1], ndata[0], ndata[1])
                    cid = result[0][0]
                    
                    cur.execute("SELECT *  FROM neighbouringcells where cid = %s AND fid = %s" % (cid,fid))
                    result = cur.fetchall()
                    if len(result) < 1: 
                        cur.execute("INSERT INTO neighbouringcells (cid,rxlev,fid) VALUES(%s,%s,%s);"%(cid,ndata[2],fid))
                        conn.commit()
                        cnt_gsmn = cnt_gsmn + 1
                        
            if wlans:
                for wlan in wlans:
                    wdata = wlan.split(',')
                    #print wdata
                    if len(wdata) > 2:
                        cur.execute("SELECT bssid FROM wlan WHERE bssid='%s';"%wdata[0])
                        result = cur.fetchall()
                        if len(result)<1:
                            cur.execute("INSERT INTO wlan (bssid, ssid, capabilitiestxt, frequency) VALUES('%s', '%s', '%s', %s) RETURNING bssid;" % (wdata[0], wdata[1].replace('"',  '\\"').  replace("'",  "\\'"), wdata[2], wdata[3]))
                            result =  cur.fetchall()
                            conn.commit()
                            print "Added new wlan: %s --> %s"%(wdata[0], wdata[1])
                        bssid = result[0][0]

                        cur.execute("SELECT * FROM wlanobservations WHERE bssid='%s' AND fid='%s';"%(bssid,fid))
                        result = cur.fetchall()
                        if len(result)<1:
                            cur.execute("INSERT INTO wlanobservations (bssid,rxlev,fid) VALUES('%s',%s,%s);"%(bssid,wdata[4],fid))
                            conn.commit()
                            cnt_wlan = cnt_wlan + 1                    
    

        print "Added %s fingerprints"%cnt_fingerprints
        print "Added %s gps-locations"%cnt_gps
        print "Added %s gsm neighbour observations"%cnt_gsmn
        print "Added %s gsm cell observations"%cnt_gsm
        print "Added %s wlan observations"%cnt_wlan

    except:
        print "ERROR %s"%(sys.exc_info()[1]);
        traceback.print_exc()


if __name__ == "__main__":
    main()
