#!/usr/bin/env python
"""


@author: Brendan Johan Lee
@contact: brendajl@simula.no
@version: 1.0
"""

import sqlite3
from optparse import OptionParser

class Fingerprint():
    """
    
    
    @author: Brendan Johan Lee
    @contact: brendajl@simula.no
    @version: 1.0
    """
    
    def __init__(self, mcc, mnc, la, cid, rxlev, lat, lon):
        """
	
        
        @param mcc: 
        @type mcc: 
        @param mnc: 
        @type mnc: 
        @param la: 
        @type la: 
        @param cid: 
        @type cid: 
        @param rxlev: 
        @type rxlev: 
        @param lat: 
        @type lat: 
        @param lon: 
        @type lon: 
        

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no
        @version: 1.0
        """
        self.mcc = mcc
        self.mnc = mnc
        self.la = la
        self.cid = cid
        self.rxlev = rxlev
        self.lat = lat
        self.lon = lon
        self.neighbors = []
        self.wlans = []

    def add_wlan(self, wlan):
        self.wlans.append(wlan)

    def add_neighbor(self, neighbor):
        self.neighbors.append(neighbor)

class Wlan(object):
    """
    
    
    @author: Brendan Johan Lee
    @contact: brendajl@simula.no
    @version: 1.0
    """
    
    def __init__(self, bssid, rxlev):
        """
	
        
        @param bssid: 
        @type bssid: 
        @param rxlev: 
        @type rxlev: 
        

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no
        @version: 1.0
        """
        self.bssid = bssid
        self.rxlev = rxlev
        

class Neighbor(object):
    """
    
    
    @author: Brendan Johan Lee
    @contact: brendajl@simula.no
    @version: 1.0
    """
    
    def __init__(self, la = None, cid = None, rxlev = None, psc = None, rscp = None):
        """
	
        
        @param cid: 
        @type cid: 
        @param rxlev: 
        @type rxlev: 
        @param psc: 
        @type psc: 
        @param rscp: 
        @type rscp: 
        

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no
        @version: 1.0
        """
        self.cid = cid
        self.rxlev = rxlev
        self.psc = psc
        self.rscp = rscp
        self.la = la

        
        
class Vault(object):
    """
    
    
    @author: Brendan Johan Lee
    @contact: brendajl@simula.no
    @version: 1.0
    """
    
    def __init__(self, dbname):
        """
	
        
        @author: Brendan Johan Lee
        @contact: brendajl@simula.no
        @version: 1.0
        """
        self.fingerprints = []
        self.dbname = dbname

    def add_fingerprint(self, fingerprint):
        self.fingerprints.append(fingerprint)

    def tosql(self):
        conn =  sqlite3.connect(self.dbname)
        cur = conn.cursor()
        for fp in self.fingerprints:
             cur.execute("INSERT INTO fingerprints (cid, la, lat, lon, mcc, mnc, rxlevel) VALUES(%s, %s, %s, %s, %s, %s, %s);" % (fp.cid, fp.la, fp.lat, fp.lon, fp.mcc, fp.mnc, fp.rxlev))
             conn.commit()
             cur.execute("SELECT last_insert_rowid();")
             result = cur.fetchall()
             for wl in fp.wlans:
                 cur.execute("INSERT INTO wlan (rxlevel, bssid, fingerprint) VALUES(%s, '%s', %s);" % (wl.rxlev, wl.bssid, result[0][0]))
             for nb in fp.neighbors:
                 if nb.psc:
                     cur.execute("INSERT INTO neighbors (psc, rscp, fingerprint) VALUES(%s, %s, %s);" % (nb.psc, nb.rscp, result[0][0]))
                 else:
                     cur.execute("INSERT INTO neighbors (la, cid, rxlevel, fingerprint) VALUES(%s, %s, %s, %s);" % (nb.la, nb.cid, nb.rxlev, result[0][0]))
             conn.commit()
        
def main():

    parser = OptionParser(version='%prog 1.0', usage="usage: %prog [options] filename", description='Convert .gsm and .wlan files to sqlite queries')
    parser.add_option('-d', '--dbname', action='store', type='string', dest='dbname', default=None, help='Set database name')
    
    (options, args) = parser.parse_args()

    if not options.dbname:
        parser.error("You must specify database name")
    
    if len(args) != 1:
        parser.error("You must specify file to be parsed")

    fingerprints = Vault(options.dbname)
        
    logfile = open(args[0], 'r')
    header = logfile.readline().rstrip('\r\n')

    for line in logfile:
        data = line.split(';')
        gps = data[0].split(',')
        gsm = data[1].split(',')
        neighbors = data[2].split('|')
        fp = Fingerprint(gsm[0], gsm[1], gsm[2], gsm[3],-int(gsm[4]), gps[3], gps[4])

        for wlan in data[3:]:
            if len(wlan) > 2: 
                wl = wlan.split(',')
                fp.add_wlan(Wlan(wl[0],-int(wl[4].strip('\n').strip('|'))))

        for nb in neighbors:
            if len(nb) > 1: 
                nbd = nb.split(',')
                if len(nbd) > 2:
                    fp.add_neighbor(Neighbor(la = nbd[0], cid = nbd[1], rxlev = -int(nbd[2].strip('\n'))))
                else:
                    fp.add_neighbor(Neighbor(psc = nbd[0], rscp = nbd[1].strip('\n')))
        
        fingerprints.add_fingerprint(fp)

    fingerprints.tosql()

if __name__ == "__main__":
    main()
