#!/usr/bin/env python

import psycopg2, sys
from optparse import OptionParser
import traceback

def main():

    parser = OptionParser(version='%prog 1.0', usage="usage: %prog [options] filename", description='Add logfile to gloc database.')
    parser.add_option('-p', '--password', action='store', type='string', dest='password', default='', help='Set database password')
    parser.add_option('-u', '--username', action='store', type='string', dest='username', default='', help='Set database username')
    parser.add_option('-n', '--hostname', action='store', type='string', dest='hostname', default='localhost', help='Set database hostname')
    parser.add_option('-d', '--dbname', action='store', type='string', dest='dbname', default='gloc', help='Set database name')
    parser.add_option('-i', '--imei', action='store', type='string', dest='imei', default='none', help='Device IMEI. Use if the logfile does not contain an IMEI. If the logfile contains an IMEI this option will be ignored.')
    parser.add_option('-t', '--filetype', action='store', type='string', dest='filetype', default='unknown', help='Set the type of file you are uploading. Legal values are gsm and wlan. If not specified the system will select the type from the filename. If this is not possible, it will try to analyze the file iteself before failing.')

    (options, args) = parser.parse_args()

    fid =  None

    if len(args) != 1:
        parser.error("You must specify file to be parsed")

    try:
        conn = psycopg2.connect("dbname='%s' user='%s' host='%s' password='%s'"%(options.dbname, options.username, options.hostname, options.password))
        cur = conn.cursor()

        infile = open(args[0], 'r')
        software = infile.readline().rstrip('\r\n')
        version = infile.readline().rstrip('\r\n')
        desc = infile.readline().rstrip('\r\n')

        v = version.split(";")
        if len(v)>1:
            version = v[0]
            options.imei = v[1]

        if options.imei == 'none':
            parser.error("No IMEI found in logfile. You must specify an IMEI")

        if options.filetype != 'wlan' and options.filetype != 'gsm':
            if args[0][-4:] == '.gsm':
                options.filetype = 'gsm'
            elif args[0][-5:] == '.wlan':
                options.filetype = 'wlan'
            elif desc[5:8] == 'mcc':
                options.filetype = 'gsm'
            elif desc[5:19] == 'horizontal_dop':
                options.filetype = 'wlan'

        if options.filetype != 'wlan' and options.filetype != 'gsm':
            parser.error("Couldn't guess filetype. Please use --filetype to specify")

        if options.filetype == 'wlan':
            desc2 = infile.readline().rstrip('\r\n')

        cur.execute("SELECT string FROM equipmentcapabilities WHERE string='%s';"%version)
        result = cur.fetchall()
        if len(result)<1:
            cur.execute("INSERT into equipmentcapabilities(string) VALUES('%s') RETURNING string;"%version)
            result = cur.fetchall()
            conn.commit()
            print "Added new equipment string: %s"%version
        equipment = result[0][0]

        cur.execute("SELECT imei FROM equipment WHERE imei=%s;"%options.imei)
        result = cur.fetchall()
        if len(result)<1:
            cur.execute("INSERT INTO equipment VALUES(%s,'%s') RETURNING imei;"%(options.imei,equipment))
            result = cur.fetchall()
            conn.commit()
            print "Added new equipment IMEI: %s"%options.imei
        imei = result[0][0]

        cnt_fingerprints = 0
        cnt_gps = 0
        cnt_gsm = 0
        cnt_wlan = 0


        for line in infile:
            entries = line.split(';')

            if len(entries)>9:
                if options.filetype == 'wlan':
                    timestamp = entries[4]
                    if entries[6] == 'NaN' or entries[9] == 'NaN':
                        fid = None
                        print "[WLAN] Skipped observation. Missing location."
                        continue
                else:
                    timestamp = entries[9]
                    if entries[11] == 'NaN' or entries[14] == 'NaN':
                        fid = None
                        print "[GSM] Skipped observation. Missing location."
                        continue
                cur.execute("SELECT fid FROM fingerprints WHERE imei=%s AND gps_time=TIMESTAMP 'epoch' + %s * INTERVAL '1 second';"%(imei,timestamp))
                result = cur.fetchall()
                if len(result)<1:
                    cur.execute("INSERT INTO fingerprints (imei,gps_time) VALUES(%s,TIMESTAMP 'epoch' + %s * INTERVAL '1 second') RETURNING fid;"%(imei,timestamp))
                    result = cur.fetchall()
                    conn.commit()
                    cnt_fingerprints = cnt_fingerprints + 1
                fid = result[0][0]

            
                cur.execute("SELECT gid FROM gps WHERE fid=%s;"%fid)
                result = cur.fetchall()
                if len(result)<1:
                    if options.filetype == 'wlan':
                        gps_values = entries[1:4]
                        gps_values.extend(entries[5:])
                    else:
                        gps_values = entries[6:9]
                        gps_values.extend(entries[10:])
                    for i in range(len(gps_values)):
                        gps_values[i] = gps_values[i].rstrip('\r\n')
                        if gps_values[i] == 'NaN':
                            gps_values[i] = 'NULL'
                    gps_values.append(fid)
                    cur.execute("INSERT INTO gps (horizontal_dop,used_satellites,vertical_dop,visible_satellites,latitude,altitude,vertical_accuracy,longitude,horizontal_accuracy,speed,heading,heading_accuracy,speed_accuracy,fid) VALUES(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s) RETURNING gid;"%(tuple(gps_values)))
                    result = cur.fetchall()
                    conn.commit()
                    cnt_gps = cnt_gps + 1
                gid = result[0][0]

            elif options.filetype == 'wlan' and fid:
                cur.execute("SELECT bssid FROM wlan WHERE bssid='%s';"%entries[4])
                result = cur.fetchall()
                if len(result)<1:
                    entries[3] = entries[3].replace("\0","")
                    entries[3] = entries[3].replace("'","\\'")
                    cur.execute("SELECT wcid FROM wlanconnectionmodes WHERE mode='%s';"%entries[5])
                    result = cur.fetchall()
                    wcm = result[0][0]
                    cur.execute("SELECT wsid FROM wlansecuritymodes WHERE mode='%s';"%entries[2])
                    result = cur.fetchall()
                    wsm = result[0][0]
                    cur.execute("INSERT INTO wlan (capabilities,beaconinterval,ssid,bssid,supportedrates,channel,connectionmode,securitymode) VALUES(%s,%s,'%s','%s',%s,%s,%s,%s) RETURNING bssid;"%(entries[0],entries[1],entries[3],entries[4],int(entries[7],16),entries[8],wcm,wsm))
                    result = cur.fetchall()
                    conn.commit()
                    print "Added new wlan: %s --> %s"%(entries[4],entries[3])
                bssid = result[0][0]

                cur.execute("SELECT * FROM wlanobservations WHERE bssid='%s' AND fid='%s';"%(bssid,fid))
                result = cur.fetchall()
                if len(result)<1:
                    cur.execute("INSERT INTO wlanobservations (bssid,rxlev,fid) VALUES('%s',%s,%s);"%(bssid,entries[6],fid))
                    conn.commit()
                    cnt_wlan = cnt_wlan + 1

            if options.filetype == 'gsm' and fid:

                cur.execute("SELECT mcc FROM mcc WHERE mcc=%s;"%entries[1])
                result = cur.fetchall()
                if len(result)<1:
                    cur.execute("INSERT INTO mcc VALUES(%s,'n/a','na');"%entries[1])
                    conn.commit()
                    print "Added new mcc: %s"%entries[1]
                    
                cur.execute("SELECT mnc FROM mnc WHERE mcc=%s AND mnc=%s;"%(entries[1],entries[2]))
                result = cur.fetchall()
                if len(result)<1:
                    cur.execute("INSERT INTO mnc VALUES(%s,%s,'n/a');"%(entries[1],entries[2]))
                    conn.commit()
                    print "Added new mcn: %s %s"%(entries[1],entries[2])

                cur.execute("SELECT la FROM la WHERE mcc=%s AND mnc=%s AND la=%s;"%(entries[1],entries[2],entries[3]))
                result = cur.fetchall()
                if len(result)<1:
                    cur.execute("INSERT INTO la VALUES(%s,%s,%s,'n/a');"%(entries[1],entries[2],entries[3]))
                    conn.commit()
                    print "Added new la: %s %s %s"%(entries[1],entries[2],entries[3])
                    
                cur.execute("SELECT cid FROM cells WHERE mcc=%s AND mnc=%s AND la=%s AND cellid=%s;"%(entries[1],entries[2],entries[3],entries[4]))
                result = cur.fetchall()
                if len(result)<1:
                    cur.execute("INSERT INTO cells (mcc,mnc,la,cellid,name) VALUES(%s,%s,%s,%s,'n/a') RETURNING cid;"%(entries[1],entries[2],entries[3],entries[4]))
                    result = cur.fetchall()
                    conn.commit()
                    print "Added new cell: %s %s %s %s"%(entries[1],entries[2],entries[3],entries[4])
                cid = result[0][0]

                cur.execute("SELECT *  FROM cellobservations where cid = %s AND fid = %s" % (cid,fid))
                result = cur.fetchall()
                if len(result) < 1: 
                    cur.execute("INSERT INTO cellobservations (cid,rxlev,fid) VALUES(%s,%s,%s);"%(cid,entries[5],fid))
                    conn.commit()
                    cnt_gsm = cnt_gsm + 1
    

        print "Added %s fingerprints"%cnt_fingerprints
        print "Added %s gps-locations"%cnt_gps
        print "Added %s gsm cell observations"%cnt_gsm
        print "Added %s wlan observations"%cnt_wlan

    except:
        print "ERROR %s"%(sys.exc_info()[1]);
        traceback.print_exc()


if __name__ == "__main__":
    main()
