#!/usr/bin/env python
"""


@author: Brendan Johan Lee
@contact: brendajl@simula.no
@version: 1.0
"""

from optparse import OptionParser
import re, math

def main():
    
    parser = OptionParser(version='%prog 1.0', usage="usage: %prog [options] filename", description='Calculate standard deviation')
    parser.add_option('-p', '--pencomp', action='store_true', dest='pencomp', default=False, help='Force pencomp output')
    parser.add_option('-a', '--pencompall', action='store_true', dest='pencompall', default=False, help='Force pencompall output')
    parser.add_option('-m', '--meanonly', action='store_true', dest='meanonly', default=False, help='Only print mean values')
    
    (options, args) = parser.parse_args()

    if len(args) < 1:
        parser.error("You must specify file to be parsed")

    matcher =  re.compile(r'diff \(meters\): (.+)$')
    numbers = []

    for arg in args: 

        try: infile = open(arg, 'r')
        except: parser.error("Error opening input file %s" % arg)
    
        for line in infile:
            result = matcher.search(line)
            numbers.append(float(result.group(1)))

    mean = sum(numbers) / len(numbers)
    difference =  [math.pow((x - mean), 2) for x in numbers]
    deviation = math.sqrt(sum(difference) / len(numbers))

    if options.meanonly:
        print mean
    elif options.pencomp:
        print mean, "|", deviation
    elif options.pencompall:
        print len(numbers),  "|", mean, "|", deviation
    else: 
        print "Measurements: ", len(numbers), " Mean: ", mean, " Standard deviation: ", deviation
    
if __name__ == "__main__":
    main()
