""" 
 Returns the closest match of the first WLAN fingerprint supplied

 @author: Brendan Johan Lee
 @contact: brendajl@simula.no

 @version: 1.0
"""

import gloclibs.server.query.basequeryfilter as basequeryfilter

class Filter(basequeryfilter.BaseQueryFilter):
    """ 
    Returns the closest match of the first WLAN fingerprint supplied

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no
    @version: 1.0
    """

    def run(self):
        """ 
        Run this filter

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no
        @version: 1.0

        @return: result of query
        @rtype: dict
        """
        fingerprint = None
        for fp in self.fingerprints:
            if fp['type']=='wlan':
                fingerprint = fp
                break

        if not fingerprint:
            return {'status':False}

        result = self.db.getClosestWLANmatch(fingerprint['bssid'],fingerprint['rxlev'])
        if not result or len(result)<1:
            return {'status':False}
        return {'status':True,'latitude':result[0][5],'longitude':result[0][7],'rxlev':result[0][1]}
