""" 
 Base class for query filters

 @author: Brendan Johan Lee
 @contact: brendajl@simula.no

 @version: 1.0
"""
import logging
import logging.config

class BaseQueryFilter:
    """ 
    Base class for query filters

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self, imei, fingerprints, db):
        """ 
        Initialize filter

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param imei: IMEI of device requesting query
        @type imei: str
        @param fingerprints: list of fingerprint(s) to be used for query
        @type fingerprints: list
        @param db: Database object to use for query
        @type db: L{glocdb.GlocDB}
        """

        self.fingerprints = fingerprints
        self.imei = imei
        self.logger = logging.getLogger("query")
        self.db = db

    def run(self):
        """ 
        Run this filter. Since base class, just return a false result

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: result of query
        @rtype: dict
        """
        return {'status':False}
