""" 
 Base class for update filters

 @author: Brendan Johan Lee
 @contact: brendajl@simula.no

 @version: 1.0
"""

import logging
import logging.config

class BaseUpdateFilter:
    """ 
    Base class for update filters

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self, imei, fingerprints, gps, db):
        """ 
        Initialize filter

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param imei: IMEI of device requesting update
        @type imei: str
        @param fingerprints: list of fingerprint(s) to be used for update
        @type fingerprints: list
        @param gps: GPS-data to be used for update
        @type gps: dict
        @param db: Database object to use for update
        @type db: L{glocdb.GlocDB}
        """

        self.fingerprints = fingerprints
        self.imei = imei
        self.gps = gps
        self.logger = logging.getLogger("update")
        self.db = db

    def run(self):
        """ 
        Run this filter. Since base class, just return no result.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: result of update
        @rtype: list
        """
        
        return {'status':'No data found'}
