#!/usr/bin/env python

import sys,  pickle
from optparse import OptionParser

def main():

    parser = OptionParser(version='%prog 1.0', usage="usage: %prog [options] file1 [file2 ... filex]", description='Converts logfile(s) to map file and xml-rpc test file.')
    parser.add_option('-m', '--mapfile', action='store', type='string', dest='mapfile', default=None, help='Output file for mapdata')
    parser.add_option('-x', '--xmlrpcfile', action='store', type='string', dest='xmlrpcfile', default=None, help='Output file for xml-rpc test data')
    parser.add_option('-a', '--algofile', action='store', type='string', dest='algofile', default=None, help='Output file for algo test data')
    parser.add_option('-i', '--imei', action='store', type='string', dest='imei', default='none', help='Device IMEI. Use if the logfile does not contain an IMEI. If the logfile contains an IMEI this option will be ignored.')
    parser.add_option('-f', '--filter', action='store', type='string', dest='filter', default=None, help='Use a different filter for xmlrpc requests')
    
    (options, args) = parser.parse_args()

    if not options.mapfile:
        parser.error("You must specify mapfile")

    if not options.xmlrpcfile:
        parser.error("You must specify xmlrpcfile")

    if not options.algofile:
        parser.error("You must specify algofile")
    
    if len(args) < 1:
        parser.error("You must specify at least one file to be parsed")

    try: mapfile = open(options.mapfile, 'w')
    except: parser.error("Error opening mapfile")

    try: xmlrpcfile = open(options.xmlrpcfile, 'w')
    except: parser.error("Error opening xmlrpcfile")

    try: algofile = open(options.algofile, 'w')
    except: parser.error("Error opening algofile")

    locations = {}

    for arg in args:
        infile = open(arg, 'r')

        info = infile.readline().rstrip('\r\n').split(",")

        if len(info)>1:
            options.imei = info[0]
            version = (',').join(info[1:])

        if options.imei == 'none':
            parser.error("No IMEI found in logfile. You must specify an IMEI")

        options.imei =  0 #quick hack
        


        for line in infile:
            wlans = None
            entries = line.rstrip('\r\n').split(';')

            
            gpsdata = entries[0].split(',')
            gsmdata = entries[1].split(',')
            neighbours = entries[2].split('|')
            if len(entries) > 3:
                wlans = entries[3:]
                wlans =  [ x.split('|')[0] for x in wlans ]

            locations[int(gpsdata[7]) / 1000] =  [gpsdata, gsmdata, neighbours, wlans]

    items = locations.items()
    items.sort()            
    trace = [(float(value[0][3]), float(value[0][4])) for key, value in items]

    pickle.dump(trace, mapfile)

    for key, item in items:
        fingerprints = []
        fingerprints.append({'type':  'gsm', 'cellid': item[1][3], 'mcc': item[1][0], 'mnc': item[1][1], 'la': item[1][2], 'rxlev': item[1][4]})

        for neighbour in item[2]:
            ndata = neighbour.split(",")
            if len(ndata) < 3 and len(ndata) > 1:
                fingerprints.append({'type':  'neighbour', 'psc': ndata[0], 'rscp': ndata[1]})
            elif len(ndata) > 2:
                fingerprints.append({'type':  'neighbour', 'cellid': ndata[1], 'mcc': item[1][0], 'mnc': item[1][1], 'la': ndata[0], 'rxlev': ndata[2]})

        if item[3]:
            for wlan in item[3]:
                wdata = wlan.split(',')
                if len(wdata) > 2:
                    fingerprints.append({'type': 'wlan', 'bssid': wdata[0], 'rxlev': wdata[4]})
                    
        xmlrpcfile.write("locationQuery('%s', %s%s)\n" % (options.imei, fingerprints, ", '%s'" % options.filter if options.filter else ""))
        algofile.write("locationQuery(%s,%s,%s,'%s', %s%s)\n" % (key, item[0][3], item[0][4], options.imei, fingerprints, ", '%s'" % options.filter if options.filter else ""))

    xmlrpcfile.close()
    mapfile.close()

if __name__ == "__main__":
    main()
