#!/usr/bin/env python

import sys,  pickle
from optparse import OptionParser

def main():

    parser = OptionParser(version='%prog 1.0', usage="usage: %prog [options] file1 [file2 ... filex]", description='Converts logfile(s) to map file and xml-rpc test file.')
    parser.add_option('-m', '--mapfile', action='store', type='string', dest='mapfile', default=None, help='Output file for mapdata')
    parser.add_option('-x', '--xmlrpcfile', action='store', type='string', dest='xmlrpcfile', default=None, help='Output file for xml-rpc test data')
    parser.add_option('-a', '--algofile', action='store', type='string', dest='algofile', default=None, help='Output file for algo test data')
    parser.add_option('-i', '--imei', action='store', type='string', dest='imei', default='none', help='Device IMEI. Use if the logfile does not contain an IMEI. If the logfile contains an IMEI this option will be ignored.')
    parser.add_option('-f', '--filter', action='store', type='string', dest='filter', default=None, help='Use a different filter for xmlrpc requests')
    
    (options, args) = parser.parse_args()

    if not options.mapfile:
        parser.error("You must specify mapfile")

    if not options.xmlrpcfile:
        parser.error("You must specify xmlrpcfile")

    if not options.algofile:
        parser.error("You must specify algofile")
    
    if len(args) < 1:
        parser.error("You must specify at least one file to be parsed")

    try: mapfile = open(options.mapfile, 'w')
    except: parser.error("Error opening mapfile")

    try: xmlrpcfile = open(options.xmlrpcfile, 'w')
    except: parser.error("Error opening xmlrpcfile")

    try: algofile = open(options.algofile, 'w')
    except: parser.error("Error opening algofile")

    locations = {}

    cnt_locations =  0
    cnt_gsm = 0
    cnt_wlan = 0

    for arg in args:
        infile = open(arg, 'r')
        software = infile.readline().rstrip('\r\n')
        version = infile.readline().rstrip('\r\n')
        desc = infile.readline().rstrip('\r\n')

        v = version.split(";")
        if len(v)>1:
            version = v[0]
            options.imei = v[1]

        if options.imei == 'none':
            parser.error("No IMEI found in logfile. You must specify an IMEI")

        filetype = None

        if arg[-4:] == '.gsm':
            filetype = 'gsm'
        elif arg[-5:] == '.wlan':
            filetype = 'wlan'
        elif desc[5:8] == 'mcc':
            filetype = 'gsm'
        elif desc[5:19] == 'horizontal_dop':
            filetype = 'wlan'

        if filetype != 'wlan' and filetype != 'gsm':
            parser.error("Couldn't guess filetype for file %s" % arg)

        if filetype == 'wlan':
            desc2 = infile.readline().rstrip('\r\n')

        for line in infile:
            entries = line.split(';')

            if len(entries)>9:
                if filetype == 'wlan':
                    timestamp = entries[4]
                    if entries[6] == 'NaN' or entries[9] == 'NaN':
                        fid = None
                        print "[WLAN] Skipped observation. Missing location."
                        continue
                else:
                    timestamp = entries[9]
                    if entries[11] == 'NaN' or entries[14] == 'NaN':
                        fid = None
                        print "[GSM] Skipped observation. Missing location."
                        continue

                if not timestamp in locations:
                    cnt_locations += 1

                    if filetype == 'wlan':
                        gps_values = entries[1:4]
                        gps_values.extend(entries[5:])
                    else:
                        gps_values = entries[6:9]
                        gps_values.extend(entries[10:])
                    for i in range(len(gps_values)):
                        gps_values[i] = gps_values[i].rstrip('\r\n')
                        if gps_values[i] == 'NaN':
                            gps_values[i] = 'NULL'

                    locations[timestamp] = [gps_values, []]
                            
            elif filetype == 'wlan':
                entries[3] = entries[3].replace("\0","")
                entries[3] = entries[3].replace("'","\\'")

                if timestamp in locations: 
                    locations[timestamp][1].append(entries)
                    cnt_wlan += 1

            if filetype == 'gsm':
                locations[timestamp][1].append(entries)
                cnt_gsm += 1

    items = locations.items()
    items.sort()
    trace = [(float(value[0][-9]), float(value[0][-6])) for key, value in items]

    pickle.dump(trace, mapfile)
        
    #TODO:  Also make xmlrpc file
    #TODO:  Remember to take the filter switch into consideration when doing xmlrpc

    for key, item in items:
        fingerprints = []
        for fp in item[1]:
             if len(fp) < 10:
                 fingerprints.append({'type': 'wlan', 'bssid': fp[4], 'rxlev': fp[6]})
             else:
                 fingerprints.append({'type':  'gsm', 'cellid': fp[4], 'mcc': fp[1], 'mnc': fp[2], 'la': fp[3], 'rxlev': fp[5]})
        xmlrpcfile.write("locationQuery('%s', %s%s)\n" % (options.imei, fingerprints, ", '%s'" % options.filter if options.filter else ""))
        algofile.write("locationQuery(%s,%s,%s,'%s', %s%s)\n" % (key, item[0][4], item[0][7], options.imei, fingerprints, ", '%s'" % options.filter if options.filter else ""))

    xmlrpcfile.close()
    mapfile.close()
                
    print "Added %s locations"%cnt_locations
    print "Added %s gsm cell observations"%cnt_gsm
    print "Added %s wlan observations"%cnt_wlan

if __name__ == "__main__":
    main()
