#!/usr/bin/env python

import psycopg2, sys
from optparse import OptionParser
import re , traceback, cmd
import LatLongUTMconversion as llu

regevaluate   = re.compile(r'.+new Coordinate\((\d+\.\d+), (\d+\.\d+)\)')
regoperator   = re.compile(r'.+Operat&oslash;r: (.+)')
regtechnology = re.compile(r'.+Teknologi: (.+)')
regfrequency  = re.compile(r'.+Frekvens: (\d+.*\d*) MHz')

class Cell:

    def __init__(self, position):
        self.latitude = position[0]
        self.longitude = position[1]
        self.transmitters = []

    def addtransmitter(self, transmitter):
        self.transmitters.append(transmitter)

    def __str__(self):
        return "latitude: %s\nlongitude: %s\n--\n%s\n"%(self.latitude, self.longitude,"\n--\n".join([str(x) for x in self.transmitters]))

class Transmitter:

    def __init__(self, operator):
        self.operator = operator

    def addtechnology(self, technology):
        self.technology = technology

    def addfrequency(self, frequency):
        self.frequency = frequency

    def __str__(self):
        return "operator: %s\ntechnology: %s\nfrequency: %s"%(self.operator, self.technology, self.frequency)

class CellNameHandler(cmd.Cmd):
    def __init__(self, cur, updated, name):
        self.cur = cur
        self.updated = updated
        self.name = name
        cmd.Cmd.__init__(self)
    
    def do_list(self, input):
        self.cur.execute("SELECT mcc,mnc,name FROM mnc;")
        result = self.cur.fetchall()
        for operator in result:
            print operator

    def help_list(self):
        print "List existing operators in database"

    def do_rename(self, input):
        values = input.split(' ')
        if len(values) < 2:
            print "Please specify mcc and mnc"
        else:
            #TODO: check that mcc and mnc are numbers
            self.cur.execute("SELECT mcc,mnc,name FROM mnc WHERE mcc=%s AND mnc=%s;"%(values[0],values[1]))
            result = self.cur.fetchall()
            if len(result)<1:
                print "No such mcc/mnc combination. Use add."
            else:
                self.cur.execute("UPDATE mnc SET name='%s' WHERE mcc=%s AND mnc=%s;"%(self.name,values[0],values[1]))
                self.updated[0] = True
                return True

    def help_rename(self):
        print "Rename an exisiting operator in database"

    def do_add(self, input):
        values = input.split(' ')
        if len(values) < 2:
            print "Please specify mcc and mnc"
        else:
            #TODO: check that mcc and mnc are numbers
            self.cur.execute("SELECT mcc,mnc,name FROM mnc WHERE mcc=%s AND mnc=%s;"%(values[0],values[1]))
            result = self.cur.fetchall()
            if len(result)>0:
                print "mcc/mnc combo exists. Use rename instead."
            else:
                self.cur.execute("SELECT mcc FROM mcc WHERE mcc=%s;"%(values[0]))
                result = self.cur.fetchall()
                if len(result)<1:
                    self.cur.execute("INSERT INTO mcc (mcc,name,iso) VALUES(%s,'n/a','na');"%(values[0]))
                self.cur.execute("INSERT INTO mnc (mcc,mnc,name) VALUES(%s,%s,'%s');"%(values[0],values[1],self.name))
                self.updated[0] = True
                return True
    
    def do_EOF(self, line):
        return True


def main():
    parser = OptionParser(version='%prog 1.0', usage="usage: %prog [options] filename", description='Add logfile to norwegian cell database.')
    parser.add_option('-p', '--password', action='store', type='string', dest='password', default='', help='Set database password')
    parser.add_option('-u', '--username', action='store', type='string', dest='username', default='', help='Set database username')
    parser.add_option('-n', '--hostname', action='store', type='string', dest='hostname', default='localhost', help='Set database hostname')
    parser.add_option('-d', '--dbname', action='store', type='string', dest='dbname', default='gloc', help='Set database name')

    (options, args) = parser.parse_args()

    if len(args) != 1:
        parser.error("You must specify file to be parsed")

    try:
        conn = psycopg2.connect("dbname='%s' user='%s' host='%s' password='%s'"%(options.dbname, options.username, options.hostname, options.password))
        cur = conn.cursor()

        infile = open(args[0], 'r')

        newcell = None
        newcells = []

        for line in infile:
            match = regevaluate.match(line)
            if match: 
                if newcell: newcells.append(newcell)
                newcell = Cell(llu.UTMtoLL(23, float(match.group(2)), float(match.group(1)), '33N'))
            match = regoperator.match(line)
            if match: transmitter = Transmitter(match.group(1))
            match = regtechnology.match(line)
            if match: transmitter.addtechnology(match.group(1))
            match = regfrequency.match(line)
            if match: 
                transmitter.addfrequency(match.group(1))
                newcell.addtransmitter(transmitter)

        if newcell: newcells.append(newcell)

        numcells = numtransmitters = numfrequencies = numtechnologies = 0

        for cell in newcells:
            try:
                cur.execute("INSERT INTO norwegiancells.sites (latitude,longitude) VALUES(%s,%s) RETURNING sid;"%(cell.latitude,cell.longitude))
                sid = cur.fetchall()[0][0]
            except:
                print "Skipping %s,%s: %s"%(cell.latitude,cell.longitude,sys.exc_info()[1])
                conn.rollback()
                continue
            numcells += 1
            for transmitter in cell.transmitters:
                failed = True
                while failed:
                    cur.execute("SELECT mcc,mnc FROM mnc WHERE name='%s';"%(transmitter.operator))
                    mccmnc = cur.fetchall()
                    if len(mccmnc)<1:
                        print "Missing operator %s.\nType help to see valid commands, or ctrl+d to terminate, then add manually and re-run script."%transmitter.operator
                        updated = [False]
                        CellNameHandler(cur,updated,transmitter.operator).cmdloop()
                        cur.execute("SELECT mcc,mnc FROM mnc WHERE name='%s';"%(transmitter.operator))
                        mccmnc = cur.fetchall()
                        if not updated[0]:
                            exit(1)
                    failed = False
                    cur.execute("SELECT * FROM norwegiancells.frequencies WHERE frequency=%s;"%transmitter.frequency)
                    result = cur.fetchall()
                    if len(result)<1:
                        cur.execute("INSERT INTO norwegiancells.frequencies (frequency) VALUES (%s) RETURNING fid;"%transmitter.frequency)
                        result = cur.fetchall()
                        numfrequencies += 1
                    fid = result[0][0]
                    cur.execute("SELECT * FROM norwegiancells.technologies WHERE technology='%s';"%transmitter.technology)
                    result = cur.fetchall()
                    if len(result)<1:
                        cur.execute("INSERT INTO norwegiancells.technologies (technology) VALUES ('%s') RETURNING tid;"%transmitter.technology)
                        result = cur.fetchall()
                        numtechnologies += 1
                    tid = result[0][0]
                    cur.execute("INSERT INTO norwegiancells.transmitters (tid,fid,mcc,mnc,sid) VALUES(%s,%s,%s,%s,%s);"%(tid,fid,mccmnc[0][0],mccmnc[0][1],sid))
                    numtransmitters += 1
            conn.commit()
        print "New cells: %s\nNew transmitters: %s\nNew frequencies: %s\nNew technologies: %s"%(numcells,numtransmitters,numfrequencies,numtechnologies)

    except:
        print "ERROR %s"%(sys.exc_info()[1]);
            
        
        

if __name__ == "__main__":
    main()

