#!/usr/bin/env python

import time, sys
import sqlite3
import sets
from optparse import OptionParser
import math

def distance(lat1,  lon1,  lat2,  lon2):
    radius =   6371 # km

    dlat =   math.radians(lat2 -  lat1)
    dlon =   math.radians(lon2 -  lon1)
    a =   math.sin(dlat /  2) *   math.sin(dlat /  2) +   math.cos(math.radians(lat1)) *   math.cos(math.radians(lat2)) *   math.sin(dlon /  2) *   math.sin(dlon /  2)
    c =   2 *   math.atan2(math.sqrt(a),   math.sqrt(1 -  a))
    d =   radius *   c
    return d *  1000

def main():

    parser = OptionParser(version='%prog 1.0', usage="usage: %prog [options] filename", description='Convert .gsm and .wlan files to sqlite queries')
    parser.add_option('-d', '--dbname', action='store', type='string', dest='dbname', default=None, help='Set database name')
    parser.add_option('-c', '--chunknumber', action='store', type='int', dest='chunck', default=None, help='Set chunknumber')
    parser.add_option('-o', '--outfile', action='store', type='string', dest='outfile', default=None, help='Set outfile')

    (options, args) = parser.parse_args()

    if not options.dbname:
        parser.error("You must specify database name")

    if options.chunck == None:
        parser.error("You must specify chunck number")

    if not options.outfile:
        parser.error("You must specify outfile")

    try: resultfile = open("%s-chunk%s" % (options.outfile, options.chunck), 'w')
    except: parser.error("Error opening outfile")    

    conn =  sqlite3.connect(options.dbname)
    cur = conn.cursor()

    t0 = time.time()

    penalty = 9500

    cur.execute("SELECT id, lat, lon FROM test LIMIT %s,1000;" % (options.chunck * 1000)); 
    tests = cur.fetchall()

    for test in tests: 

        finals = {}
    
        cur.execute("SELECT bssid, wlan.rxlevel FROM wlan WHERE wlan.fingerprint = %s;" % test[0]);
        result = cur.fetchall()

        cnt1 = len(result)

        if cnt1 > 0: 
    
            for r in result:
                cur.execute("select fingerprint, wlan.rxlevel from wlan, base where bssid ='%s' and wlan.fingerprint = base.id;" % r[0])
                result2 = cur.fetchall()

                for r2 in result2:
                    if r2[0] not in finals:
                        finals[r2[0]] = [(r[1] - r2[1]) ** 2, 0]
                    else:
                        finals[r2[0]][0] += (r[1] - r2[1]) ** 2

            for k in finals.keys():
                cur.execute("select count(*) from wlan where fingerprint=%s;" % k)
                pen = cur.fetchall()
                finals[k][1] = abs(cnt1 - pen[0][0])
                finals[k][0] -= (finals[k][1] * penalty)

            if len(finals) > 0: 
            
                winner = sorted(finals.items(), key = lambda e: e[1][0], reverse = True)[0]

                cur.execute("SELECT lat, lon FROM base WHERE id =%s;" % winner[0]); 
                winnerpos = cur.fetchall()

                resultfile.write("testing: %s winner: %s score: %s penalty: %s true: %s,%s estimated: %s,%s diff (meters): %s\n" % (test[0], winner[0], winner[1][0], winner[1][1], test[1], test[2], winnerpos[0][0], winnerpos[0][1], distance(test[1], test[2], winnerpos[0][0], winnerpos[0][1]) ))
#             print winnerpos
#             print test[1:]
#             print test[0]
#             print winner[0]

#             print " -- "

    #PRINT ID OF WINNER AND WHAT BEING TESTED so we can use later for limiting

    resultfile.close()
    try: timefile = open("time-%s-chunk%s" % (options.outfile, options.chunck), 'w')
    except: parser.error("Error opening timefile")
    timefile.write("%s" % (time.time() - t0))
    timefile.close()
    t0 = time.time()

if __name__ == "__main__":
    main()
