#!/usr/bin/env python

import time, sys
import sqlite3
import sets
from optparse import OptionParser
import math, Polygon

def distance(lat1,  lon1,  lat2,  lon2):
    radius =   6371 # km

    dlat =   math.radians(lat2 -  lat1)
    dlon =   math.radians(lon2 -  lon1)
    a =   math.sin(dlat /  2) *   math.sin(dlat /  2) +   math.cos(math.radians(lat1)) *   math.cos(math.radians(lat2)) *   math.sin(dlon /  2) *   math.sin(dlon /  2)
    c =   2 *   math.atan2(math.sqrt(a),   math.sqrt(1 -  a))
    d =   radius *   c
    return d *  1000

def main():

    parser = OptionParser(version='%prog 1.0', usage="usage: %prog [options] filename", description='Convert .gsm and .wlan files to sqlite queries')
    parser.add_option('-d', '--dbname', action='store', type='string', dest='dbname', default=None, help='Set database name')
    parser.add_option('-o', '--outfile', action='store', type='string', dest='outfile', default=None, help='Set outfile')

    (options, args) = parser.parse_args()

    if not options.dbname:
        parser.error("You must specify database name")

    if not options.outfile:
        parser.error("You must specify outfile")

    try: resultfile = open(options.outfile, 'w')
    except: parser.error("Error opening outfile")    

    conn =  sqlite3.connect(options.dbname)
    cur = conn.cursor()

    t0 = time.time()

    cur.execute("SELECT id, lat, lon FROM test;")
    tests = cur.fetchall()

    for test in tests: 

        areas = []

        cur.execute("SELECT mcc, mnc, la, cid FROM test WHERE test.id = %s;" % test[0]);
        resultx = cur.fetchall()

        cnt1 = len(resultx)

        area = None
    
        if cnt1 > 0:

            cur.execute("SELECT area FROM simpleareas WHERE mcc=%s AND mnc=%s AND la=%s AND cid=%s" % (resultx[0][0], resultx[0][1], resultx[0][2], resultx[0][3]))
            result2 = cur.fetchall()

            if result2: 
                if len(result2) > 0: 
                    newpol = [(float(y[0]), float(y[1])) for y in [x.split(',') for x in result2[0][0].split(';')]]
                    if len(newpol) > 3: 
                        area = Polygon.Polygon(newpol)

            if area: 

                bx = 0
                by = 0
                pcnt = 0

                for point in area[0]:
                    bx += point[0]
                    by += point[1]
                    pcnt += 1

                bx =  bx / pcnt
                by =  by / pcnt

                resultfile.write("testing: %s true: %s,%s estimated: %s,%s diff (meters): %s\n" % (test[0], test[1], test[2], bx, by, distance(test[1], test[2], bx, by)))

    resultfile.close()
    try: timefile = open("time-%s" % (options.outfile), 'w')
    except: parser.error("Error opening timefile")
    timefile.write("%s" % (time.time() - t0))
    timefile.close()
    t0 = time.time()

if __name__ == "__main__":
    main()
