#!/usr/bin/env python

import time, sys
import sqlite3
import sets
from optparse import OptionParser
import math, Polygon

def distance(lat1,  lon1,  lat2,  lon2):
    radius =   6371 # km

    dlat =   math.radians(lat2 -  lat1)
    dlon =   math.radians(lon2 -  lon1)
    a =   math.sin(dlat /  2) *   math.sin(dlat /  2) +   math.cos(math.radians(lat1)) *   math.cos(math.radians(lat2)) *   math.sin(dlon /  2) *   math.sin(dlon /  2)
    c =   2 *   math.atan2(math.sqrt(a),   math.sqrt(1 -  a))
    d =   radius *   c
    return d *  1000

def main():

    parser = OptionParser(version='%prog 1.0', usage="usage: %prog [options] filename", description='Convert .gsm and .wlan files to sqlite queries')
    parser.add_option('-d', '--dbname', action='store', type='string', dest='dbname', default=None, help='Set database name')
    parser.add_option('-o', '--outfile', action='store', type='string', dest='outfile', default=None, help='Set outfile')

    (options, args) = parser.parse_args()

    if not options.dbname:
        parser.error("You must specify database name")

    if not options.outfile:
        parser.error("You must specify outfile")

    try: resultfile = open(options.outfile, 'w')
    except: parser.error("Error opening outfile")    

    conn =  sqlite3.connect(options.dbname)
    cur = conn.cursor()

    t0 = time.time()

    cur.execute("SELECT id, lat, lon FROM test;")
    tests = cur.fetchall()

    for test in tests: 

        areas = []

        cur.execute("SELECT * FROM neighbors WHERE neighbors.fingerprint = %s AND (la != 0 OR psc IS NOT NULL);" % test[0]);
        result = cur.fetchall()

        cur.execute("SELECT mcc, mnc, la, cid, rxlevel FROM test WHERE test.id = %s;" % test[0]);
        resultx = cur.fetchall()

        cnt1 = len(result)

        cur.execute("SELECT bssid, wlan.rxlevel FROM wlan WHERE wlan.fingerprint = %s;" % test[0]);
        resulty = cur.fetchall()

        cnt2 =  len(resulty)

        if cnt1 > 0 or cnt2 > 0:

            if cnt1 > 0: 

                for r in result:

                    if r[0] != None:
                        cur.execute("SELECT area FROM neighboringareas WHERE la=%s AND cid=%s AND rxlevel=%s;" % (r[0], r[1], r[2]))
                        result2 = cur.fetchall()
                        if len(result2) > 0: 
                            newpol = [(float(y[0]), float(y[1])) for y in [x.split(',') for x in result2[0][0].split(';')]]
                            if len(newpol) > 3: 
                                areas.append(Polygon.Polygon(newpol))
                        
                    else:
                        cur.execute("select area FROM neighboringareas3g WHERE psc=%s AND rscp=%s;" % (r[4], r[5]))
                        result2 = cur.fetchall()
                        if len(result2) > 0: 
                            newpol = [(float(y[0]), float(y[1])) for y in [x.split(',') for x in result2[0][0].split(';')]]
                            if len(newpol) > 3: 
                                areas.append(Polygon.Polygon(newpol))

            if cnt2 > 0:
                for r in resulty:
                    cur.execute("SELECT area  FROM wlanareas WHERE bssid = '%s' AND rxlevel = %s" % (r[0], r[1]))
                    result2 = cur.fetchall()

                    if len(result2) > 0: 
                        newpol = [(float(y[0]), float(y[1])) for y in [x.split(',') for x in result2[0][0].split(';')]]
                        if len(newpol) > 3: 
                            areas.append(Polygon.Polygon(newpol))
                
            cur.execute("SELECT area FROM cellareas WHERE mcc=%s AND mnc=%s AND la=%s AND cid=%s AND rxlevel=%s" % (resultx[0][0], resultx[0][1], resultx[0][2], resultx[0][3], resultx[0][4]))
            result2 = cur.fetchall()

            if len(result2) > 0: 
                newpol = [(float(y[0]), float(y[1])) for y in [x.split(',') for x in result2[0][0].split(';')]]
                if len(newpol) > 3: 
                    areas.append(Polygon.Polygon(newpol))

            if len(areas) > 0: 
                area = areas[0]
                
                points = {}
                cnt = 1
                penalty = 0

                for ar in areas[1:]:
                    if ar[0][0] == ar[0][-1] and ar[0][1] == ar[0][-2]: 
                        for point in ar[0]:
                            if area.covers(Polygon.Polygon([(point), (point), (point), (point)])):
                                if point not in points:
                                    points[point] = 0
                                else:
                                    points[point] += 1
                            else:
                                penalty += 1
                    else:
                        cnt += 1
                        if area.overlaps(ar):
                            area = ar & area
                        else:
                            penalty += 1

                bx = 0
                by = 0
                pcnt = 0

                for point in area[0]:
                    bx += point[0]
                    by += point[1]
                    pcnt += 1

                bx =  bx / pcnt
                by =  by / pcnt

                resultfile.write("testing: %s penalty: %s true: %s,%s estimated: %s,%s diff (meters): %s\n" % (test[0], penalty, test[1], test[2], bx, by, distance(test[1], test[2], bx, by)))


    resultfile.close()
    try: timefile = open("time-%s" % (options.outfile), 'w')
    except: parser.error("Error opening timefile")
    timefile.write("%s" % (time.time() - t0))
    timefile.close()
    t0 = time.time()

if __name__ == "__main__":
    main()
